/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __DRV_EEPROM_24C02_H
#define __DRV_EEPROM_24C02_H

/* Includes ------------------------------------------------------------------*/
#include "mg32l003.h"
#include "mg32l003_i2c.h"

/* Exported types ------------------------------------------------------------*/
/**
  * @brief  EEPROM communication stage enum definition
  */
typedef enum 
{
  EEPROM_SEND               = 0,
  EEPROM_ADDR_BEFORE_RX     = 1,  /* MCU should send register address before receiving data from eeprom */
  EEPROM_RECEIVE            = 2,
} EEPROM_Stage;

/**
  * @brief  EEPROM communication error enum definition
  */
typedef enum 
{
  EEPROM_NONE                   = 0,
  EEPROM_SEND_ERROR             = 1,
  EEPROM_SEND_RIGHT             = 2,
  EEPROM_ADDR_BEFORE_RX_ERROR   = 3,
  EEPROM_ADDR_BEFORE_RX_RIGHT   = 4,
  EEPROM_RECEIVE_ERROR          = 5,
  EEPROM_RECEIVE_RIGHT          = 6,
} EEPROM_Status;

/* Exported constants --------------------------------------------------------*/
#define EEPROM_DEVICE_ADDRESS    0XA0
#define EEPROM_REGISTER_ADDRESS  0X44
/* Exported macro ------------------------------------------------------------*/
#define EEPROM_GET_ERROR(EEPROM_STAGE)  ((EEPROM_STAGE == EEPROM_SEND) ?(EEPROM_SEND_ERROR): \
                                        ((EEPROM_STAGE == EEPROM_ADDR_BEFORE_RX) ?(EEPROM_ADDR_BEFORE_RX_ERROR):(EEPROM_RECEIVE_ERROR)))
/* Exported functions --------------------------------------------------------*/

void eeprom_24c02_init(void);

#endif /* __DRV_EEPROM_24C02_H */
