/* Includes ------------------------------------------------------------------*/
#include "mg32l003.h"
#include "bsp_uart1.h"
#include "drv_eeprom_24c02.h"
#include <stdio.h>

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
EEPROM_Status eeprom_status;
EEPROM_Stage eeprom_stage;

uint8_t pRData[2] = {0X00, 0X00};
uint8_t pWData[3] = {EEPROM_REGISTER_ADDRESS, 0X33, 0X66};
uint32_t target_size = 3;
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

void delay(volatile uint32_t n)
{
  while (n--)
    ;
}

/* You shoule change the pWData every time to check the I2C */
int main(void)
{
  SystemCoreClockUpdate();

  uart1_init(24000000, 115200);
  eeprom_24c02_init();

  /* Infinite loop */
  while (1)
  {
    /* if the data are writen into eeprom */
    if (eeprom_status == EEPROM_SEND_RIGHT)
    {
      /* Set eeprom error status */
      eeprom_status = EEPROM_ADDR_BEFORE_RX_ERROR;
      /* Set stage into addressing before receiving data */
      eeprom_stage = EEPROM_ADDR_BEFORE_RX;
      /* Change the target size */
      target_size = 1;
      /* Insert a delay time */
      delay(200000);
      /* Start addressing before receiving data from 24c02 */
      I2C_Start(ENABLE);
    }
    else if (eeprom_status == EEPROM_ADDR_BEFORE_RX_RIGHT)
    {
      /* Set eeprom error status */
      eeprom_status = EEPROM_RECEIVE_ERROR;
      /* Set stage into receiving data from eeprom */
      eeprom_stage = EEPROM_RECEIVE;
      /* Change the target size */
      target_size = 2;
      /* Start receiving data from 24c02 */
      I2C_Start(ENABLE);
    }
    else if (eeprom_status == EEPROM_RECEIVE_RIGHT)
    {
      if ((pRData[0] == pWData[1]) && (pRData[1] == pWData[2]))
      {
        printf("Write and read right\r\n");
        printf("Read from eeprom data0 =0x%x,data1=0x%x\r\n", pRData[0], pRData[1]);
      }
      else
      {
        printf("Write and read wrong\r\n");
        printf("Read from eeprom data0 =0x%x,data1=0x%x\r\n", pRData[0], pRData[1]);
      }
      eeprom_status = EEPROM_NONE;
    }
    else
    {
      ;
    }
  }
}

#ifdef USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif
