/* Includes ------------------------------------------------------------------*/
#include "mg32l003.h"
#include "bsp_uart1.h"
#include <stdio.h>

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
uint8_t test_flag = 0;
/* Private function prototypes -----------------------------------------------*/
void MY_VC_Init(void);

/* Private functions ---------------------------------------------------------*/

void delay(volatile uint32_t n)
{
  while (n--)
    ;
}

/* Case1: If the input voltage in PC5 is lower than 1.25 v,
          then uart1 print "Filter output is low".
   Case2: If the input voltage in PC5 is higher than 1.25 v,
          then uart1 print "Filter output is high".
   Note1: Interrupt occured when the filter output voltage is a rising edge as the configuration,
          which means that you should give PC5 a rising edge cross 1.25v to trigger VC interrupt.
   Note2: The uart1 will print message every 0.5 s
*/
int main(void)
{
  SystemCoreClockUpdate();

  uart1_init(24000000, 115200);
  MY_VC_Init();

  /* Infinite loop */
  while (1)
  {
    if (test_flag == 1)
    {
      test_flag = 0;
      printf("Filter Interrupt!\r\n");
    }

    if (VC_GetOutputStatus() != RESET)
      printf("Filter output is High\r\n");
    else
      printf("Filter output is Low\r\n");

    delay(1550000);
  }
}

void MY_VC_Init(void)
{
  VC_InitTypeDef VC_InitStruct;

  RCC_APBPeriphClockCmd(RCC_APBPeriph_LVDVC, ENABLE);
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);

  GPIO_Init(GPIOC, GPIO_Pin_5, GPIO_OTYPE_PP | GPIO_PUPD_NOPULL | GPIO_SPEED_HIGH);
  GPIO_SetAFPin(GPIOC, GPIO_Pin_5, GPIO_AF15);

  VC_InitStruct.V25div = VC_2P4_V25DIV;
  VC_InitStruct.Filter_Clk = VC_FILTER_CLK_PCLK;
  VC_InitStruct.NegPinSel = VC_INPUT_V25DIV;
  VC_InitStruct.PosPinSel = VC_INPUT_1;
  VC_InitStruct.Filter_Enable = VC_FILTER_ENABLE;
  VC_InitStruct.Fliter_Num = 0x10;
  VC_Init(&VC_InitStruct);

  VC_ITConfig(VC_RISE_INT, ENABLE);
  VC_ITCmd(ENABLE);
  NVIC_SetPriority(VC_IRQn, 1);
  NVIC_EnableIRQ(VC_IRQn);
  VC_Cmd(ENABLE);
}

void VC_IRQHandler(void)
{
  if (VC_GetITStatus() != RESET)
  {
    test_flag = 1;
    VC_ClearITPendingBit();
  }
}

#ifdef USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif
